use Test::More tests => 1;

BEGIN {
	use_ok( 'POE::Component::Server::Chargen' );
}

diag( "Testing POE::Component::Server::Chargen-$POE::Component::Server::Chargen::VERSION, POE-$POE::VERSION, Perl $], $^X" );

